@@ -1,8 +1,8 @@
-from abc import abstractmethod
+"""
+Implementation of metrics for (mulit-horizon) timeseries forecasting.
+"""
 from typing import Dict, List, Union
 
-from sklearn.base import BaseEstimator
-
 import torch
 from torch import nn
 import torch.nn.functional as F
@@ -13,19 +13,28 @@ from pytorch_lightning.metrics import TensorMetric
 import scipy.stats
 
 
-class Metric(TensorMetric, metaclass=abc.ABCMeta):
+class Metric(TensorMetric):
     """
     Base metric class that has basic functions that can handle predicting quantiles and operate in log space
 
     Other metrics should inherit from this base class
     """
 
-    def __init__(self, name: str, quantiles: List[float] = [0.5], reduction="mean"):
+    def __init__(self, name: str = None, quantiles: List[float] = [0.5], reduction="mean"):
+        """
+        Initialize metric
+
+        Args:
+            name (str): metric name. Defaults to class name.
+            quantiles (List[float], optional): quantiles for probability range. Defaults to [0.5].
+            reduction (str, optional): Reduction, "none", "mean" or "sqrt-mean". Defaults to "mean".
+        """
         self.quantiles = quantiles
         self.reduction = reduction
+        if name is None:
+            name = self.__class__.__name__
         super().__init__(name)
 
-    @abstractmethod
     def forward(self, y_pred: torch.Tensor, y_actual: torch.Tensor) -> torch.Tensor:
         """
         Abstract method that calcualtes metric
@@ -39,7 +48,7 @@ class Metric(TensorMetric, metaclass=abc.ABCMeta):
         Returns:
             torch.Tensor: metric value on which backpropagation can be applied
         """
-        pass
+        raise NotImplementedError()
 
     def to_prediction(self, y_pred: torch.Tensor) -> torch.Tensor:
         """
@@ -88,13 +97,13 @@ class MultiHorizonMetric(Metric):
         Returns:
             torch.Tensor: loss/metric as a single number for backpropagation
         """
-        pass
+        raise NotImplementedError()
 
     def forward(self, y_pred: Dict[str, torch.Tensor], target: Union[torch.Tensor, rnn.PackedSequence]) -> torch.Tensor:
         """
         Forward method of metric that handles masking of values.
 
-        Do not override this method but :py:ref:`~loss` instead
+        Do not override this method but :py:meth:`~loss` instead
 
         Args:
             y_pred (Dict[str, torch.Tensor]): network output
@@ -109,7 +118,7 @@ class MultiHorizonMetric(Metric):
             # batch sizes reside on the CPU by default -> we need to bring them to GPU
             lengths = lengths.to(target.device)
         else:
-            lengths = torch.LongTensor([target.size(1)], device=target.device).expand(target.size(0))
+            lengths = torch.ones(target.size(0), device=target.device, dtype=torch.long) * target.size(1)
         assert not target.requires_grad
 
         # calculate loss with "none" reduction
@@ -130,7 +139,7 @@ class MultiHorizonMetric(Metric):
             mask = mask.unsqueeze(-1)
         # reduce to one number
         if self.reduction == "none":
-            loss = losses.masked_fill(mask, "nan")
+            loss = losses.masked_fill(mask, float("nan"))
         else:
             if self.reduction == "mean":
                 losses = losses.masked_fill(mask, 0.0)
